@@ -1,11 +1,11 @@
 package Net::DNS::RR::NXT;
 
-# $Id: NXT.pm,v 1.6 2002/12/20 10:20:21 olaf Exp $
+# $Id: NXT.pm 318 2005-05-30 16:36:52Z olaf $
 
 use strict;
 use vars qw(@ISA $VERSION);
 use Carp;
-
+use bytes;
 use Net::DNS;
 use Net::DNS::Packet;
 
@@ -13,10 +13,11 @@ use Net::DNS::Packet;
 use Carp;
 
 @ISA = qw(Net::DNS::RR);
-$VERSION = do { my @r=(q$Revision: 1.6 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
+$VERSION = do { my @r=(q$Revision: 318 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
 
 sub new {
     my ($class, $self, $data, $offset) = @_;
+    carp "The NXT RR is depricated as of RFC3755, please refrain from using this record";
     
     if ($self->{"rdlength"} > 0) {
 	my($nxtdname,$nxtoffset) = 
@@ -32,12 +33,12 @@ sub new {
 	$self->{"typelist"} = join " " 
 	    ,  _typebm2typestr($typebm);
     }
-    
     return bless $self, $class;
 }
 
 sub new_from_string {
     my ($class, $self, $string) = @_;
+    carp "The NXT RR is depricated as of RFC3755";
     if ($string) {
 	$string =~ tr/()//d;
 	$string =~ s/;.*$//mg;
@@ -55,6 +56,20 @@ sub new_from_string {
 }
 
 
+
+
+#sub is_optin {
+#    my $self =shift;
+#    return 1 if $self->{"typelist"}!~/NXT/;
+#    0;
+#}
+
+#sub set_optin {
+#    my $self =shift;
+#    $self->{"typelist"}=~s/NXT//;
+#    1;
+#}
+
 sub rdatastr {
 	my $self = shift;
 	my $rdatastr;
@@ -152,12 +167,17 @@ Net::DNS::RR::NXT - DNS NXT resource record
 
 =head1 SYNOPSIS
 
-C<use Net::DNS::RR>;
+C<use Net::DNS::RR;>
 
 =head1 DESCRIPTION
 
+B<NOTE: THE NXT RR has been deprecated! Use NSEC instead.>
+
+
+
 Class for DNS Address (NXT) resource records.
 
+
 =head1 METHODS
 
 =head2 nxtdname
@@ -186,7 +206,7 @@ on daily basis.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001  RIPE NCC.  Author Olaf M. Kolkman <net-dns-sec@ripe.net>
+Copyright (c) 2001-2005  RIPE NCC.  Author Olaf M. Kolkman <olaf@net-dns.org>
 
 All Rights Reserved
 
@@ -208,12 +228,14 @@ OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 Based on, and contains, code by Copyright (c) 1997 Michael Fuhr.
 
-
-
 =head1 SEE ALSO
 
+L<http://www.net-dns.org/> 
+
 L<perl(1)>, L<Net::DNS>, L<Net::DNS::Resolver>, L<Net::DNS::Packet>,
 L<Net::DNS::Header>, L<Net::DNS::Question>, L<Net::DNS::RR>,
 RFC 2435 Section 5
 
+
+
 =cut
