@@ -16,17 +16,19 @@ use MIME::Base64;
 use Math::BigInt;
 use Time::Local;
 
+@ISA = qw(Net::DNS::SEC);
 
 require Exporter;
 
-$VERSION = do { my @r=(q$Revision: 527 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
+$VERSION = do { my @r=(q$Revision: 1129 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
 
 sub new {
     my ($class,  $key_file) = @_;
     my $self={};
     my    ($Modulus,$PublicExponent,$PrivateExponent,$Prime1,
 	   $Prime2,$Exponent1,$Exponent2,$Coefficient,
-	   $prime_p,$subprime_q,$base_g,$private_val_x,$public_val_y);
+	   $prime_p,$subprime_q,$base_g,$private_val_x,$public_val_y,
+	   $Created, $Publish, $Activate, $Revoke, $Unpublish, $Delete);
     
 
     bless ($self,$class);
@@ -51,11 +53,11 @@ sub new {
     
     while (<KEYFH>) {
 	if (/Private-key-format: (v\d*\.\d*)/) {
-	    if ($1 ne "v1.2") {
-		croak "Private Key Format not regognized";
+	    if ($1 ne "v1.2" && $1 ne "v1.3") {
+		croak "Private Key Format not recognized";
 	    }
 	}elsif	    (/^Algorithm:\s*(\d*)/) {
-	    if ($1 != 1 && $1 != 3 && $1 != 5) {
+	    if ($1 != 1 && $1 != 3 && $1 != 5 && $1 !=6 && $1 != 7 && $1 != 8 && $1 != 10 ) {
 		croak "Key $key_file algorithm is not RSA or DSA (those are the only implemented algorithms) ";
 	    }
 	    
@@ -87,11 +89,27 @@ sub new {
 	    $private_val_x=decode_base64($1);
 	} elsif (/^Public_value\(y\):\s*(\S+)/) { 
 	    $public_val_y=decode_base64($1);
+	} elsif (/^Created\(y\):\s*(\S+)/) { 
+	    $Created=$1;
+	} elsif (/^Publish\(y\):\s*(\S+)/) { 
+	    $Publish=$1;
+	} elsif (/^Activate\(y\):\s*(\S+)/) { 
+	    $Activate=$1;
+	} elsif (/^Revoke\(y\):\s*(\S+)/) { 
+	    $Revoke=$1;
+	} elsif (/^Unpublish\(y\):\s*(\S+)/) { 
+	    $Unpublish=$1;
+	} elsif (/^Delete\(y\):\s*(\S+)/) { 
+	    $Delete=$1;
 	}
     }
     close(KEYFH);
 
-    if ($self->{"algorithm"} == 1 || $self->{"algorithm"} == 5) {  #RSA
+    if ($self->{"algorithm"} == 1
+     || $self->{"algorithm"} == 5
+     || $self->{"algorithm"} == 7
+     || $self->{"algorithm"} == 8
+     || $self->{"algorithm"} == 10) {  #RSA
       $self->{'privatekey'}=Crypt::OpenSSL::RSA-> 
 	  new_key_from_parameters(
 	      $Modulus,
@@ -106,7 +124,6 @@ sub new {
 
       # Trying to determine the keytag
 
-
       my $keytag_from_data1=$self->dump_rsa_keytag(256,1);
       my $keytag_from_data2=$self->dump_rsa_keytag(257,1);
       if (($self->{"keytag"} != $keytag_from_data1) &&
@@ -117,7 +134,8 @@ sub new {
 	  return(0);
       }
       
-    }elsif ($self->{"algorithm"} == 3){  #DSA
+    }elsif ($self->{"algorithm"} == 3
+	|| $self->{"algorithm"} == 6 ){  #DSA
 	my $private_dsa = Crypt::OpenSSL::DSA->new();
 	$private_dsa->set_p($prime_p);
 	$private_dsa->set_q($subprime_q);
@@ -126,6 +144,17 @@ sub new {
 	$private_dsa->set_pub_key($public_val_y);
 	$self->{"privatekey"}=$private_dsa;
     }
+
+    if (defined($Created)) {
+	# new fields in v1.3
+	$self->{'created'} = $Created;
+	$self->{'publish'} = $Publish;
+	$self->{'activate'}  = $Activate;
+	$self->{'revoke'}    = $Revoke;
+	$self->{'unpublish'} = $Unpublish;
+	$self->{'delete'}    = $Delete;
+    }
+
     return $self;
 
 }
@@ -133,12 +162,6 @@ sub new {
 
 
 
-sub algorithm {
-    my $self=shift;
-    return $self->{'algorithm'};
-}
-
-
 sub privatekey {
     my $self=shift;
     return $self->{'privatekey'};
@@ -158,6 +181,24 @@ sub signame {
 }
 
 
+sub created {
+    my $self=shift;
+    return $self->{'created'} if (exists($self->{'created'}));
+}
+
+
+sub publish {
+    my $self=shift;
+    return $self->{'publish'} if (exists($self->{'publish'}));
+}
+
+
+sub activate {
+    my $self=shift;
+    return $self->{'activate'} if (exists($self->{'activate'}));
+}
+
+
 # Little helper function to put a BigInt into a binary (unsigned,
 #network order )
 
@@ -186,11 +227,15 @@ sub signame {
 
 
 sub new_rsa_priv {
-    my ($class,  $keyblob,$signame,$flags) = @_;
+    my ($class,  $keyblob,$signame,$flags,$algorithm) = @_;
     my $self={};
     bless ($self,$class);
     $self->{"signame"}=$signame;
-    $self->{"algorithm"}=5;
+    if (defined($algorithm)) {
+    	$self->{"algorithm"} = $self->algorithm($algorithm);
+    } else {
+    	$self->{"algorithm"} = 5;
+    }
     $self->{"flags"}=$flags;
     $self->{'privatekey'}=Crypt::OpenSSL::RSA->  
 	new_private_key($keyblob);
@@ -205,7 +250,7 @@ sub  dump_rsa_priv {
     my ( $Modulus,$PublicExponent, $PrivateExponent, $Prime1, $Prime2, $Exponent1,
 	 $Exponent2,$Coefficient )=$self->{"privatekey"}->get_key_parameters;
     my $string="Private-key-format: v1.2\n";
-    $string .= "Algorithm: 5 (RSASHA1)\n";
+    $string .= "Algorithm: " . $self->{"algorithm"} . " (" . $self->algorithm("mnemonic") . ")\n";
     
     if (defined $Modulus 
 	&& defined $PublicExponent 
@@ -263,10 +308,11 @@ sub dump_rsa_keytag{
     # feature that a non-zero second argument to this function will
     # _not_ set the flag.
     $self->{"flags"}=$flags unless shift; 
-
+    my $alg=$self->{"algorithm"};
+    return () unless ($alg ==1 || $alg ==5 || $alg == 7 || $alg ==8 || $alg ==10);
     my $key=$self->dump_rsa_pub;
     return ()  unless $key;
-    my $tmprr=Net::DNS::RR->new("tmp  IN DNSKEY $flags 3 5 $key");
+    my $tmprr=Net::DNS::RR->new("tmp  IN DNSKEY $flags 3 $alg  $key");
     return $tmprr->keytag;
 }
 
@@ -286,11 +332,17 @@ sub generate_rsa {
     my $size=shift;
     $size=1024 if !defined ($size);
     my $good_entropy=shift;
+    my $algorithm=shift;
+    if (defined($algorithm)) {
+	$algorithm = $class->algorithm($algorithm);
+    } else {
+    	$algorithm = 5;
+    }
     my $self={};
     bless ($self,$class);
 
     $self->{"signame"}=$name;  
-    $self->{"algorithm"}= 5; #  Force non-string 
+    $self->{"algorithm"}= $algorithm; #  Force non-string 
     if (defined($good_entropy)){
 	Crypt::OpenSSL::Random::random_seed($good_entropy);
 	  Crypt::OpenSSL::RSA->import_random_seed();
@@ -351,14 +403,19 @@ or Crypt::OpenSSL::DSA object. This is really only relevant to the
 Net::DNS::RR::SIG class.
 
 
-=head2  algorithm, keytag, signame
+=head2  algorithm, keytag, signame, created, publish, activate
  
  $private->algorithm
  $private->keytag
  $private->signame
+ $private->created
+ $private->publish
+ $private->activate
 
 Returns components as determined from the filename and needed by
-Net::DNS::RR::RRSIG.
+Net::DNS::RR::RRSIG.  The 'created', 'publish' and 'activate'
+components are only available in version 1.3 or higher formatted
+files.
 
 
 =head1 RSASHA1 specific helper functions
@@ -370,7 +427,7 @@ from X509 format.
 
 Constructor method.
 
- my $private=Net::DNS::SEC::Private->new_rsa_private($keyblob,$domain,$flag);
+ my $private=Net::DNS::SEC::Private->new_rsa_private($keyblob,$domain,$flag,$algorithm);
 
 Creates a Net::DNS::SEC::Private object from the supplied string.  For
 the object to be useful you will have to provide the "domain" name for
@@ -407,8 +464,8 @@ Returns an empty string on failure.
     my $flags=257;   # SEP key.
     my $keytag=$private->dump_rsa_keytag($flags);
 
-This function will calculate the keytag assuming that key is a used
-with the RSASHA1 signing algorithm and with the DNSKEY flags as input.
+This function will calculate the keyt with the value of the DNSKEY
+flags as input.
 
 The flags field may be needed in case it was not specified when the
 key was created. If the object allready knows it's flags vallue the
@@ -425,7 +482,7 @@ can be read with the read_rsa_private method.)
 
 =head2 generate_rsa
 
-    my $keypair=Net::DNS::SEC::Private->generate_rsa("example.com",$flag,1024,$random);
+    my $keypair=Net::DNS::SEC::Private->generate_rsa("example.com",$flag,1024,$random, $algorithm);
 prin $newkey->dump_rsa_priv;
 print $newkey->dump_rsa_pub();
 
